@@ -1,16 +1,34 @@
 package Catalyst::Response;
 
-use strict;
-use base 'Class::Accessor::Fast';
+use Moose;
+use HTTP::Headers;
 
-__PACKAGE__->mk_accessors(qw/cookies body headers location status/);
+with 'MooseX::Emulate::Class::Accessor::Fast';
 
-*output = \&body;
+has cookies   => (is => 'rw', default => sub { {} });
+has body      => (is => 'rw', default => '', lazy => 1, predicate => 'has_body');
+has location  => (is => 'rw');
+has status    => (is => 'rw', default => 200);
+has finalized_headers => (is => 'rw', default => 0);
+has headers   => (
+  is      => 'rw',
+  handles => [qw(content_encoding content_length content_type header)],
+  default => sub { HTTP::Headers->new() },
+  required => 1,
+  lazy => 1,
+);
+has _context => (
+  is => 'rw',
+  weak_ref => 1,
+  handles => ['write'],
+  clearer => '_clear_context',
+);
 
-sub content_encoding { shift->headers->content_encoding(@_) }
-sub content_length   { shift->headers->content_length(@_) }
-sub content_type     { shift->headers->content_type(@_) }
-sub header           { shift->headers->header(@_) }
+sub output { shift->body(@_) }
+
+sub code   { shift->status(@_) }
+
+no Moose;
 
 =head1 NAME
 
@@ -20,6 +38,7 @@ Catalyst::Response - stores output responding to the current client request
 
     $res = $c->response;
     $res->body;
+    $res->code;
     $res->content_encoding;
     $res->content_length;
     $res->content_type;
@@ -48,6 +67,14 @@ you might want to use a L<IO::Handle> type of object (Something that implements
 in the same fashion), or a filehandle GLOB. Catalyst
 will write it piece by piece into the response.
 
+=head2 $res->has_body
+
+Predicate which returns true when a body has been set.
+
+=head2 $res->code
+
+Alias for $res->status.
+
 =head2 $res->content_encoding
 
 Shortcut for $res->headers->content_encoding.
@@ -133,19 +160,46 @@ sub redirect {
     return $self->location;
 }
 
+=head2 $res->location
+
+Sets or returns the HTTP 'Location'.
+
 =head2 $res->status
 
 Sets or returns the HTTP status.
 
     $c->response->status(404);
+
+$res->code is an alias for this, to match HTTP::Response->code.
     
 =head2 $res->write( $data )
 
 Writes $data to the output stream.
 
+=head2 meta
+
+Provided by Moose
+
+=head2 $res->print( @data )
+
+Prints @data to the output stream, separated by $,.  This lets you pass
+the response object to functions that want to write to an L<IO::Handle>.
+
 =cut
 
-sub write { shift->{_context}->write(@_); }
+sub print {
+    my $self = shift;
+    my $data = shift;
+
+    defined $self->write($data) or return;
+
+    for (@_) {
+        defined $self->write($,) or return;
+        defined $self->write($_) or return;
+    }
+    
+    return 1;
+}
 
 =head1 AUTHORS
 
@@ -158,4 +212,6 @@ it under the same terms as Perl itself.
 
 =cut
 
+__PACKAGE__->meta->make_immutable;
+
 1;
